
#include "TChain.h"
#include "TFile.h"
#include "TH1F.h"
#include "TH2F.h"
#include "TH3F.h"
#include "TMath.h"
#include "TF1.h"
#include "TCanvas.h"
#include "TStopwatch.h"
#include <iostream>
#include <iomanip>

using namespace std;


// --- Declaration of the functions -----------------------------------------
TH2F *calcStatErr    (TH2F* eff, TH2F* statErreffi, Int_t binX, Int_t binY);
void  printProgress  (Int_t ct,Int_t total,Int_t st);
Bool_t checkPhi      (Float_t phi, Float_t range);




void readAccEffPos()
{
   /////////////////////////////////////////////////////////////////////
    // This macro reads the ntuples for yield,acceptance and
    // efficiency of positrons/electrons created by HPidSingleLeptonEff
    // and projects the efficiency/acceptance matrices (phi,theta,mom)
    // for single particle efficiencies.
    //
    // TO DO:
    // 1. select "ep" or "em" for e+/e- calculations
    // 2. setup the filelist
    // 3. set outputfile name
    // 4. check cuts (only in efficiency loop)
    // 5. run the compiled program
    //
    // The matrices can be smoothed afterwards using smoothMatrix.C
    //
    // to make use of the projections, use ROOT5 instead of buggy 
    // ROOT4, where Projection3D() does not work correctly!!!
    // 
    // usage:
    // compile the macro with ".L readAccEff.C++" on the ROOT interpreter
    ////////////////////////////////////////////////////////////////////

   gROOT->SetBatch();

   TStopwatch* timer = new TStopwatch();
   timer->Reset();
   timer->Start();

   TString outputFile = "matrixEffPosJ_Jan04.root";
   TString particle   = "Pos";
   Int_t stepProgress = 1;  // print progress of loop [%]

   //-------------------------------------------------------------------
   // define histogram binning
   Int_t binPhi       = 90;
   Int_t binTheta     = 30;
   Int_t binP         = 100;
   //-------------------------------------------------------------------

   //-------------------------------------------------------------------
   // define histogram range
   Float_t minPhi   =   0.; // [deg]
   Float_t maxPhi   = 360.; // [deg]
   Float_t minTheta = 0.  ; // [deg]
   Float_t maxTheta = 90. ; // [deg]
   Float_t minP     = 0.;
   Float_t maxP     = 2000.;
   Float_t edgePhi  = 4.;   // [deg]

   Double_t weight = 0.;

   //-------------------------------------------------------------------
   // define histogram binning
   Float_t* binsTheta = new Float_t [binTheta + 1];
   binsTheta[0] = minTheta;
   for(Int_t t=1; t<binTheta; t++)
   {
      Float_t stepTheta = (maxTheta-minTheta)/binTheta;
      binsTheta[t] = t*stepTheta + minTheta;
   }
   binsTheta[binTheta] = maxTheta;

   //-------------------------------------------------------------------
   // define histogram binning
   Float_t* binsPhi = new Float_t [binPhi + 1];
   binsPhi[0] = minPhi;
   for(Int_t t=1; t<binPhi; t++)
   {
      Float_t stepPhi = (maxPhi-minPhi)/binPhi;
      binsPhi[t] = t*stepPhi + minPhi;
   }
   binsPhi[binPhi] = maxPhi;

   //-------------------------------------------------------------------
   // define histogram binning
   Float_t* binsP = new Float_t [binP + 1];
   binsP[0] = minP;
   for(Int_t t=1; t<binP; t++)
   {
      Float_t stepP = (maxP-minP)/binP;
      binsP[t] = t*stepP + minP;
   }
   binsP[binP] = maxP;
   const Int_t nbinsP = binP;

   //-------------------------------------------------------------------
   //-------------------------------------------------------------------
   cout << "Output file   : " <<outputFile.Data()<<endl;
   cout << "     Phi step : " <<setw(8)<< (maxPhi   - minPhi  ) / binPhi    << " [deg]"   << endl;
   cout << "   Theta step : " <<setw(8)<< (maxTheta - minTheta) / binTheta  << " [deg]"   << endl;


   gROOT->Reset();

   //-------------------------------------------------------------------
   // the name of the ntuple for e+ and e- are different

   // the name of the ntuple
   TChain* effPos  = new TChain("effPos");
   TChain* accePos = new TChain("accePos");
   TChain* allPos  = new TChain("simYieldPos");
   //-------------------------------------------------------------------

   //-------------------------------------------------------------------
   // add here your ntuple files which should
   // be analyzed


   effPos ->Add("/home/mweber/rich15/simul/richdigi/jan04/jan04_white_posi_999_eff_nt.root");
   accePos->Add("/home/mweber/rich15/simul/richdigi/jan04/jan04_white_posi_999_eff_nt.root");
   allPos ->Add("/home/mweber/rich15/simul/richdigi/jan04/jan04_white_posi_999_eff_nt.root");
   //-------------------------------------------------------------------

   //-------------------------------------------------------------------
   // Declaration of leaves types (efficiency)
   Float_t         pRecon;
   Float_t         pGeant;
   Float_t         thetaRecon;
   Float_t         thetaGeant;
   Float_t         phiRecon;
   Float_t         phiGeant;
   Float_t         ptGeant;
   Float_t         isCutNb;
   Float_t         commonTrack;
   Float_t         bitFieldPassedCuts;
   Float_t         angleHadronFitted;
   Float_t         angleHadronNonFitted;
   Float_t         angleLeptonFitted;
   Float_t         angleLeptonNonFitted;
   Float_t         isRing;
   Float_t         innerChi2;
   Float_t         rkChi2;
   Float_t         metaMatchQality;
   Float_t         metaMatchRK;
   Float_t         vertX;
   Float_t         vertY;
   Float_t         vertZ;
   Float_t         outerChi2;
   Float_t         isHiddenLepton;
   Float_t         beta;
   Float_t         pairVertx;
   Float_t         pairVerty;
   Float_t         pairVertz;
   Float_t         opang;
   Float_t         pReconPartner;
   Float_t         pidRecPartner;
//    Float_t         isFake;
//    Float_t         isBadTrack;
//    Float_t         isBadRkMetaMatch;
   //-------------------------------------------------------------------

   //-------------------------------------------------------------------
   //Declaration of leaves types (acceptance)
   Float_t         p;
   Float_t         theta;
   Float_t         phi;
   Float_t         pt;
   //-------------------------------------------------------------------

   //-------------------------------------------------------------------
   //Declaration of leaves types (simulated yield)
   Float_t         pYield;
   Float_t         thetaYield;
   Float_t         phiYield;
   Float_t         ptYield;
   //-------------------------------------------------------------------


   //-------------------------------------------------------------------
   // Set branch addresses (efficiency)
   effPos->SetBranchAddress("pRecon",&pRecon);
   effPos->SetBranchAddress("pGeant",&pGeant);
   effPos->SetBranchAddress("thetaRecon",&thetaRecon);
   effPos->SetBranchAddress("thetaGeant",&thetaGeant);
   effPos->SetBranchAddress("phiRecon",&phiRecon);
   effPos->SetBranchAddress("phiGeant",&phiGeant);
   effPos->SetBranchAddress("ptGeant",&ptGeant);
   effPos->SetBranchAddress("isCutNb",&isCutNb);
   effPos->SetBranchAddress("commonTrack",&commonTrack);
   effPos->SetBranchAddress("bitFieldPassedCuts",&bitFieldPassedCuts);
   effPos->SetBranchAddress("angleHadronFitted",&angleHadronFitted);
   effPos->SetBranchAddress("angleHadronNonFitted",&angleHadronNonFitted);
   effPos->SetBranchAddress("angleLeptonFitted",&angleLeptonFitted);
   effPos->SetBranchAddress("angleLeptonNonFitted",&angleLeptonNonFitted);
   effPos->SetBranchAddress("isRing",&isRing);
   effPos->SetBranchAddress("innerChi2",&innerChi2);
   effPos->SetBranchAddress("rkChi2",&rkChi2);
   effPos->SetBranchAddress("metaMatchQality",&metaMatchQality);
   effPos->SetBranchAddress("metaMatchRK",&metaMatchRK);
   effPos->SetBranchAddress("vertX",&vertX);
   effPos->SetBranchAddress("vertY",&vertY);
   effPos->SetBranchAddress("vertZ",&vertZ);
   effPos->SetBranchAddress("outerChi2",&outerChi2);
   effPos->SetBranchAddress("isHiddenLepton",&isHiddenLepton);
   effPos->SetBranchAddress("beta",&beta);
   effPos->SetBranchAddress("pairVertx",&pairVertx);
   effPos->SetBranchAddress("pairVerty",&pairVerty);
   effPos->SetBranchAddress("pairVertz",&pairVertz);
   effPos->SetBranchAddress("opang",&opang);
   effPos->SetBranchAddress("pReconPartner",&pReconPartner);
   effPos->SetBranchAddress("pidRecPartner",&pidRecPartner);
   effPos->SetBranchAddress("isFake",&isFake);
   effPos->SetBranchAddress("isBadTrack",&isBadTrack);
   effPos->SetBranchAddress("isBadRkMetaMatch",&isBadRkMetaMatch);
   //-------------------------------------------------------------------

   //-------------------------------------------------------------------
   // Set branch addresses (acceptance)
   accePos->SetBranchAddress("p",&p);
   accePos->SetBranchAddress("theta",&theta);
   accePos->SetBranchAddress("phi",&phi);
   accePos->SetBranchAddress("pt",&pt);
   //-------------------------------------------------------------------

   //-------------------------------------------------------------------
   // Set branch addresses (simulated yield)
   allPos->SetBranchAddress("p",&pYield);
   allPos->SetBranchAddress("theta",&thetaYield);
   allPos->SetBranchAddress("phi",&phiYield);
   allPos->SetBranchAddress("pt",&ptYield);
   //-------------------------------------------------------------------

   //-------------------------------------------------------------------
   // get the number of entrie sin each chain
   Long64_t nentriesEff   = effPos  ->GetEntries();
   Long64_t nentriesAcc   = accePos ->GetEntries();
   Long64_t nentriesYield = allPos  ->GetEntries();
   //-------------------------------------------------------------------

   Int_t nbytes = 0;

   //-------------------------------------------------------------------
   // histogram definition

   // --------   Y I E L D   ------------------
   TH3F *yield3D     = new TH3F(Form("yield3D%s",particle.Data()),
                                Form("yield3D%s",particle.Data()),
                                binPhi,binsPhi,binTheta,binsTheta,nbinsP,binsP);
   yield3D->SetDirectory(0);
   yield3D->Sumw2();
   yield3D->GetZaxis()->SetRange(1,nbinsP);
   TH3F *yield3Dw    = new TH3F(Form("yield3Dw%s",particle.Data()),
                                Form("yield3Dw%s",particle.Data()),
                                binPhi,binsPhi,binTheta,binsTheta,nbinsP,binsP);
   yield3Dw->SetDirectory(0);
   yield3Dw->Sumw2();
   yield3Dw->GetZaxis()->SetRange(1,nbinsP);
   TH3F *yield3Dnb     = new TH3F(Form("yield3Dnb%s",particle.Data()),
                                  Form("yield3Dnb%s",particle.Data()),
                                  binPhi,binsPhi,binTheta,binsTheta,nbinsP,binsP);
   yield3Dnb->SetDirectory(0);
   yield3Dnb->Sumw2();
   yield3Dnb->GetZaxis()->SetRange(1,nbinsP);
   TH3F *yield3Dwnb    = new TH3F(Form("yield3Dwnb%s",particle.Data()),
                                  Form("yield3Dwnb%s",particle.Data()),
                                  binPhi,binsPhi,binTheta,binsTheta,nbinsP,binsP);
   yield3Dwnb->SetDirectory(0);
   yield3Dwnb->Sumw2();
   yield3Dwnb->GetZaxis()->SetRange(1,nbinsP);

   // --------   A C C E P T A N C E   ------------------
   TH3F *acc3D       = new TH3F(Form("acce3D%s",particle.Data()),
                                Form("acce3D%s",particle.Data()),
                                binPhi,binsPhi,binTheta,binsTheta,nbinsP,binsP);
   acc3D->SetDirectory(0);
   acc3D->Sumw2();
   acc3D->GetZaxis()->SetRange(1,nbinsP);
   TH3F *acc3Dw       = new TH3F(Form("acce3Dw%s",particle.Data()),
                                 Form("acce3Dw%s",particle.Data()),
                                 binPhi,binsPhi,binTheta,binsTheta,nbinsP,binsP);
   acc3Dw->SetDirectory(0);
   acc3Dw->Sumw2();
   acc3Dw->GetZaxis()->SetRange(1,nbinsP);
   TH3F *acc3Dnb       = new TH3F(Form("acce3Dnb%s",particle.Data()),
                                  Form("acce3Dnb%s",particle.Data()),
                                  binPhi,binsPhi,binTheta,binsTheta,nbinsP,binsP);
   acc3Dnb->SetDirectory(0);
   acc3Dnb->Sumw2();
   acc3Dnb->GetZaxis()->SetRange(1,nbinsP);
   TH3F *acc3Dwnb       = new TH3F(Form("acce3Dwnb%s",particle.Data()),
                                   Form("acce3Dwnb%s",particle.Data()),
                                   binPhi,binsPhi,binTheta,binsTheta,nbinsP,binsP);
   acc3Dwnb->SetDirectory(0);
   acc3Dwnb->Sumw2();
   acc3Dwnb->GetZaxis()->SetRange(1,nbinsP);

   // --------   I D E A L   E F F   ------------------
   TH3F *eff3DIdeal  = new TH3F(Form("effi3DIdeal%s",particle.Data()),
                                Form("effi3DIdeal%s",particle.Data()),
                                binPhi,binsPhi,binTheta,binsTheta,nbinsP,binsP);
   eff3DIdeal->SetDirectory(0);
   eff3DIdeal->Sumw2();
   eff3DIdeal->GetZaxis()->SetRange(1,nbinsP);
   TH3F *eff3DIdealnb  = new TH3F(Form("effi3DIdealnb%s",particle.Data()),
                                  Form("effi3DIdealnb%s",particle.Data()),
                                  binPhi,binsPhi,binTheta,binsTheta,nbinsP,binsP);
   eff3DIdealnb->SetDirectory(0);
   eff3DIdealnb->Sumw2();
   eff3DIdealnb->GetZaxis()->SetRange(1,nbinsP);
   TH3F *eff3DIdealw  = new TH3F(Form("effi3DIdealw%s",particle.Data()),
                                 Form("effi3DIdealw%s",particle.Data()),
                                 binPhi,binsPhi,binTheta,binsTheta,nbinsP,binsP);
   eff3DIdealw->SetDirectory(0);
   eff3DIdealw->Sumw2();
   eff3DIdealw->GetZaxis()->SetRange(1,nbinsP);
   TH3F *eff3DIdealwnb  = new TH3F(Form("effi3DIdealwnb%s",particle.Data()),
                                   Form("effi3DIdealwnb%s",particle.Data()),
                                   binPhi,binsPhi,binTheta,binsTheta,nbinsP,binsP);
   eff3DIdealwnb->SetDirectory(0);
   eff3DIdealwnb->Sumw2();
   eff3DIdealwnb->GetZaxis()->SetRange(1,nbinsP);

   // --------   R E A L   E F F   ------------------
   TH3F *eff3DReal   = new TH3F(Form("effi3DReal%s",particle.Data()),
                                Form("effi3DReal%s",particle.Data()),
                                binPhi,binsPhi,binTheta,binsTheta,nbinsP,binsP);
   eff3DReal->SetDirectory(0);
   eff3DReal->Sumw2();
   eff3DReal->GetZaxis()->SetRange(1,nbinsP);
   TH3F *eff3DRealnb   = new TH3F(Form("effi3DRealnb%s",particle.Data()),
                                  Form("effi3DRealnb%s",particle.Data()),
                                  binPhi,binsPhi,binTheta,binsTheta,nbinsP,binsP);
   eff3DRealnb->SetDirectory(0);
   eff3DRealnb->Sumw2();
   eff3DRealnb->GetZaxis()->SetRange(1,nbinsP);
   TH3F *eff3DRealw   = new TH3F(Form("effi3DRealw%s",particle.Data()),
                                 Form("effi3DRealw%s",particle.Data()),
                                 binPhi,binsPhi,binTheta,binsTheta,nbinsP,binsP);
   eff3DRealw->SetDirectory(0);
   eff3DRealw->Sumw2();
   eff3DRealw->GetZaxis()->SetRange(1,nbinsP);
   TH3F *eff3DRealwnb   = new TH3F(Form("effi3DRealwnb%s",particle.Data()),
                                   Form("effi3DRealwnb%s",particle.Data()),
                                   binPhi,binsPhi,binTheta,binsTheta,nbinsP,binsP);
   eff3DRealwnb->SetDirectory(0);
   eff3DRealwnb->Sumw2();
   eff3DRealwnb->GetZaxis()->SetRange(1,nbinsP);
   //-------------------------------------------------------------------


   //-------------------------------------------------------------------
   // Fill efficiency histograms

   cout<<"loop efficiency : "<<flush;
   for(Long64_t k = 0; k < nentriesEff; k ++)
   {
      nbytes += effPos->GetEntry(k);

      printProgress(k,nentriesEff,stepProgress);

      //-------------------------------------------------------------------
      // Apply standard cuts

      // remove close neigbour
//       if(angleLeptonFitted    < 9.) continue;
//      if(angleLeptonNonFitted < 9.) continue;

      // remove fakes, etc...
//       if ( kTRUE == isFake ||
//            kTRUE == isBadTrack ||
//            kTRUE == isBadRkMetaMatch )
//          continue;

 
      //-------------------------------------------------------------------
      //   I D E A L  matrices here

      weight = 6.6e-3 * (exp(-6.8e-03*pGeant)+(1.2e-03)*exp(-3.7e-04*pGeant));

      eff3DIdealnb ->Fill(phiGeant,thetaGeant,pGeant);
      eff3DIdealwnb->Fill(phiGeant,thetaGeant,pGeant, weight);

     // check phi
      if( kTRUE ==  checkPhi(phiGeant, edgePhi) )
      {
         //-------------------------------------------------------------------
         // in addition, border condition is checked here
         eff3DIdeal ->Fill(phiGeant,thetaGeant,pGeant);
         eff3DIdealw->Fill(phiGeant,thetaGeant,pGeant, weight);
      }


      //-------------------------------------------------------------------
      //   R E A L  matrices here

      weight = 6.6e-3 * (exp(-6.8e-03*pRecon)+(1.2e-03)*exp(-3.7e-04*pRecon));

      eff3DRealnb ->Fill(phiRecon, thetaRecon, pRecon);
      eff3DRealwnb->Fill(phiRecon, thetaRecon, pRecon, weight);

      // check phi
      if( kTRUE ==  checkPhi(phiRecon, edgePhi) )
      {
         //-------------------------------------------------------------------
         // in addition, border condition is checked here
         eff3DReal ->Fill(phiRecon, thetaRecon, pRecon);
         eff3DRealw->Fill(phiRecon, thetaRecon, pRecon, weight);
      }
      //-------------------------------------------------------------------

   }
   cout<<endl;

   //-------------------------------------------------------------------
   // Fill acceptance histograms
   cout<<"loop acceptance : "<<flush;
   for(Long64_t k = 0; k < nentriesAcc; k ++)
   {
      nbytes += accePos->GetEntry(k);
      printProgress(k,nentriesAcc,stepProgress);

      weight = 6.6e-3 * (exp(-6.8e-03*p)+(1.2e-03)*exp(-3.7e-04*p));

      acc3Dnb ->Fill(phi,theta,p);
      acc3Dwnb->Fill(phi,theta,p,weight);

      // check phi
      if( kFALSE == checkPhi(phi, edgePhi) ) continue;

      acc3D ->Fill(phi,theta,p);
      acc3Dw->Fill(phi,theta,p,weight);
   }
   cout<<endl;
   //-------------------------------------------------------------------

   //-------------------------------------------------------------------
   // Fill simulated yield histograms
   cout<<"loop yield      : "<<flush;
   for(Long64_t k = 0; k < nentriesYield; k ++)
   {
      nbytes += allPos->GetEntry(k);
      printProgress(k,nentriesYield,stepProgress);

      weight = 6.6e-3 * (exp(-6.8e-03*pYield)+(1.2e-03)*exp(-3.7e-04*pYield));

      yield3Dnb ->Fill(phiYield,thetaYield,pYield);
      yield3Dwnb->Fill(phiYield,thetaYield,pYield,weight);

      // check phi
      if( kFALSE == checkPhi(phiYield, edgePhi) ) continue;

      yield3D ->Fill(phiYield,thetaYield,pYield);
      yield3Dw->Fill(phiYield,thetaYield,pYield,weight);
   }
   cout<<endl;

   //-------------------------------------------------------------------


   //-------------------------------------------------------------------
   //-------------------------------------------------------------------
   // Create efficiency histograms
   TH3F* cpAcc3D       = (TH3F*)acc3D->Clone("cpAcc3D");
   cpAcc3D ->SetDirectory(0);
   cpAcc3D ->Sumw2();
   cpAcc3D ->GetZaxis()->SetRange(1,nbinsP);

   TH3F* cpAcc3Dw      = (TH3F*)acc3Dw->Clone("cpAcc3Dw");
   cpAcc3Dw ->SetDirectory(0);
   cpAcc3Dw ->Sumw2();
   cpAcc3Dw ->GetZaxis()->SetRange(1,nbinsP);

   TH3F* cpAcc3Dnb     = (TH3F*)acc3Dnb->Clone("cpAcc3Dnb");
   cpAcc3Dnb ->SetDirectory(0);
   cpAcc3Dnb ->Sumw2();
   cpAcc3Dnb ->GetZaxis()->SetRange(1,nbinsP);

   TH3F* cpAcc3Dwnb    = (TH3F*)acc3Dwnb->Clone("cpAcc3Dwnb");
   cpAcc3Dwnb ->SetDirectory(0);
   cpAcc3Dwnb ->Sumw2();
   cpAcc3Dwnb ->GetZaxis()->SetRange(1,nbinsP);


   acc3D        ->Divide(yield3D);
   acc3Dw       ->Divide(yield3Dw);
   acc3Dnb      ->Divide(yield3Dnb);
   acc3Dwnb     ->Divide(yield3Dwnb);

   eff3DIdeal   ->Divide(cpAcc3D);
   eff3DIdealw  ->Divide(cpAcc3Dw);
   eff3DIdealnb ->Divide(cpAcc3Dnb);
   eff3DIdealwnb->Divide(cpAcc3Dwnb);

   eff3DReal    ->Divide(cpAcc3D);
   eff3DRealw   ->Divide(cpAcc3Dw);
   eff3DRealnb  ->Divide(cpAcc3Dnb);
   eff3DRealwnb ->Divide(cpAcc3Dwnb);


   //-------------------------------------------------------------------
   // writing histograms to output
   TFile *file = new TFile(outputFile.Data(),"recreate");
   file->cd();
   cout << "writing output" << endl;

   acc3D        ->Write();
   acc3Dw       ->Write();
   acc3Dnb      ->Write();
   acc3Dwnb     ->Write();

   eff3DIdeal   ->Write();
   eff3DIdealw  ->Write();
   eff3DIdealnb ->Write();
   eff3DIdealwnb->Write();

   eff3DReal    ->Write();
   eff3DRealw   ->Write();
   eff3DRealnb  ->Write();
   eff3DRealwnb ->Write();

   file->Close();
   //-------------------------------------------------------------------

   timer->Stop();

   cout<<"Calculation Time: "<<timer->RealTime()<<" s "<<endl;
   delete timer;
}


//-------------------------------------------------------------
TH2F *calcStatErr(TH2F* eff, TH2F* statErreffi, Int_t binX, Int_t binY)
{
   for(Int_t j = 1; j < binY + 1; j ++)
   {
      for(Int_t i = 1; i < binX + 1; i ++)
      {
         Float_t count =  eff->GetBinContent(i,j);
         if(count > 0) statErreffi->SetBinContent(i,j, sqrt(count) / count * 100.);
         else statErreffi->SetBinContent(i,j,0.);
      }
   }
   return statErreffi;
}
//-------------------------------------------------------------


//-------------------------------------------------------------
// This function will
Bool_t checkPhi(Float_t phi, Float_t range)
{
   if(TMath::Abs(fmod(phi,60.)) < range ||
      TMath::Abs(fmod(phi,60.)) > 60-range)
      return kFALSE;
   else
      return kTRUE;
}
//-------------------------------------------------------------


//-------------------------------------------------------------
void printProgress(Int_t ct,Int_t total,Int_t st) {

   Int_t frac   =  (Int_t) ((( ct     /(Float_t)total) ) * 100);
   Int_t frac_1 =  (Int_t) ((((ct - 1)/(Float_t)total) ) * 100);

   if( frac % st == 0 &&  (frac != frac_1) )
   {
      if(frac_1 / st > 0 ) cout<<"\b\b\b\b\b"<<flush;
      cout<<setw(3)<<frac<<" %"<<flush;

   }
}
//-------------------------------------------------------------
